/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: demux.h,v 1.12 2000/12/08 00:51:12 f1rmb Exp $
 */

#ifndef HAVE_DEMUX_H
#define HAVE_DEMUX_H

#include "buffer.h"
#include "xine.h"
#include "input/input_plugin.h"

#define DEMUX_OK       0
#define DEMUX_FINISHED 1

/*
 * these are the functions any xine demuxer must provide
 */

typedef struct demux_functions_s
{
  /*
   * returns 1 if demux is able to handle given MRL
   */

  int (*demux_can_handle) (char *MRL);

  /*
   * start demux thread
   * pos : 0..65535
   */

  void (*demux_start) (input_plugin_t *input_plugin, fifo_buffer_t *bufVideo, 
		       fifo_buffer_t *bufAudio, off_t pos) ;
  
  /*
   * stop & kill demux thread
   */

  void (*demux_stop) () ;

  /*
   * returns DEMUX_OK or  DEMUX_FINISHED 
   */

  int (*demux_get_status) () ;

  /*
   * selects audio channel
   */
  
  void (*demux_select_audio_channel) (int nChannel);

} demux_functions_t ;

demux_functions_t *init_demux_mpeg ();

demux_functions_t *init_demux_mpeg_block ();

demux_functions_t *init_demux_avi ();

#endif

