/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: gui_main.c,v 1.34 2000/12/13 00:00:59 f1rmb Exp $
 *
 * XINE-PANEL 
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>
#include <X11/keysym.h>
#include <Imlib.h>
#include "gui_main.h"
#include "gui_list.h"
#include "gui_button.h"
#include "gui_label.h"
#include "gui_dnd.h"
#include "gui_slider.h"
#include "gui_parseskin.h"
#include "xine.h"
#include "../video_out/video_out.h"
#include "configfile.h"

/*
 * global variables
 */
static Window          gui_panel_win;
static gui_move_t      gui_move; 
static widget_list_t  *gui_widget_list;
static ImlibImage     *gui_bg_image;                 /* background image */
extern ImlibData      *gImlib_data;
extern Display        *gDisplay;
extern vo_driver_t    *gVideoDriver;
extern Window          gVideoWin;

static widget_t        *gui_slider_play;
static widget_t        *gui_slider_mixer;

static char            gui_filename[1024];
static widget_t       *gui_title_label;
static char            gui_runtime[20];
static widget_t       *gui_runtime_label;
static int             gui_panel_visible;
static int             gui_running;

static char            gui_audiochan[20];
static widget_t       *gui_audiochan_label;

/*
 * simple gui playlist
 */
static char           *gui_playlist[100];
static int             gui_playlist_num;
static int             gui_playlist_cur;
static int             gui_ignore_status=0;

static int             cursor_visible;

extern int nVideoHours ;
extern int nVideoMins ;
extern int nVideoSecs ;

#define NEXT 1
#define PREV 2
extern int curAudioTrack;
extern uint32_t xine_debug;

#define MAX_UPDSLD 200
static int update_slider;

/*
 * Callback-functions for widget-button click
 */

void gui_exit (widget_t *w, void *data) 
{
  xprintf(VERBOSE|GUI, "xine-panel: EXIT\n");
  xine_exit(); 
  gui_running = 0;
}

void gui_play (widget_t *w, void *data)
{
  /* TODO: find slider in widget_list. now its created as the last widget .. so its easy to find it ;) */

  /*
    FIXME
  widget_t *widget = (widget_t*) gui_list_last_content(gui_widget_list->l);
  
  if (widget->widget_type == WIDGET_TYPE_SLIDER) {
    slider_make_step(widget);
  }
  */
  xprintf(VERBOSE|GUI, "xine-panel: PLAY\n");
  xine_play ( gui_filename, 0 );
}

void gui_dndcallback (char *filename) {
  strcpy (gui_filename, filename);
  set_label (gui_title_label, gui_filename);
  paint_widget_list (gui_widget_list);

  gui_play (NULL, NULL); 
  update_slider = MAX_UPDSLD;
}

void gui_stop (widget_t *w, void *data)
{
  gui_ignore_status = 1;
  xine_stop ();
  vo_set_logo_mode (1); 
  gui_ignore_status = 0;
  slider_reset(gui_widget_list, gui_slider_play);
}

void gui_pause (widget_t *w, void *data)
{
  xine_pause ();
}

void gui_nextprev(widget_t *w, void *data) {
  
  if(((int)data) == NEXT) {
    gui_ignore_status = 1;
    xine_stop ();
    gui_ignore_status = 0;
    gui_status_callback (XINE_STOP);
  }
  else if(((int)data) == PREV) {
    gui_ignore_status = 1;
    xine_stop ();
    gui_playlist_cur--;
    if ((gui_playlist_cur>=0) && (gui_playlist_cur < gui_playlist_num)) {
      strcpy(gui_filename, gui_playlist [gui_playlist_cur]);
      set_label (gui_title_label, gui_filename);
      paint_label (gui_title_label, gui_panel_win, gui_widget_list->gc);
      xine_play ( gui_filename, 0 );
    } else {
      vo_set_logo_mode (1); 
      gui_playlist_cur = 0;
    }
    gui_ignore_status = 0;
  }

  update_slider = MAX_UPDSLD;
}

void gui_toggle_fullscreen(widget_t *w, void *data) {

  vo_set_fullscreen (!vo_is_fullscreen ());
  vo_display_cursor(gui_panel_visible);
  cursor_visible = gui_panel_visible;
  if (gui_panel_visible)  {
    XRaiseWindow (gDisplay, gui_panel_win);
    XSetTransientForHint (gDisplay, gui_panel_win, gVideoWin);
  }
}

void gui_toggle_aspect(void) {

  vo_set_aspect (vo_get_aspect () + 1);
  if (gui_panel_visible)  {
    XRaiseWindow (gDisplay, gui_panel_win);
    XSetTransientForHint (gDisplay, gui_panel_win, gVideoWin);
  }
}

void gui_toggle_panel_visibility (widget_t *w, void *data) {

  if (gui_panel_visible) {

    gui_panel_visible = 0;
    XUnmapWindow (gDisplay, gui_panel_win);

  } else {

    gui_panel_visible = 1;
    XMapRaised(gDisplay, gui_panel_win); 
    XSetTransientForHint (gDisplay, gui_panel_win, gVideoWin);
    update_slider = MAX_UPDSLD;

  }

  vo_display_cursor(gui_panel_visible);
  cursor_visible = gui_panel_visible;
  config_file_set_int ("open_panel", gui_panel_visible);
}

void gui_change_audio_channel(widget_t *w, void *data) {
  
  if(((int)data) == NEXT) {
    xine_select_audio_channel((curAudioTrack + 1));
  }
  else if(((int)data) == PREV) {
    if(curAudioTrack)
      xine_select_audio_channel((curAudioTrack - 1));
  }
  
  sprintf (gui_audiochan, "%3d", curAudioTrack);
  set_label (gui_audiochan_label, gui_audiochan);
  paint_label (gui_audiochan_label, gui_panel_win, gui_widget_list->gc); 
}

void gui_set_current_position (int pos) {
  gui_ignore_status = 1;
  xine_stop ();
  xine_play ( gui_filename, pos);
  gui_ignore_status = 0;
  update_slider = MAX_UPDSLD;
}

void gui_slider(widget_t *w, void *data, int pos) {

  if(w == gui_slider_play) {
    gui_set_current_position (pos);
    if(xine_get_status() != XINE_PLAY)
      slider_reset(gui_widget_list, gui_slider_play);
  }
  else if(w == gui_slider_mixer) {
    // TODO
  }
  else
    xprintf(VERBOSE|GUI, "unknown widget slider caller\n");

}

void gui_nothing (void)
{
	// Just do nothing ...
}

/*
 * Function for Loading Images, using ImLib
 */
 
gui_image_t *gui_load_image(const char *image)
{
  ImlibImage *img;
  gui_image_t *i;

  i = (gui_image_t *) malloc(sizeof(gui_image_t));

  if( !( img = Imlib_load_image(gImlib_data, (char *)image) ) ) {
	  fprintf(stderr, "xine-panel: couldn't find image %s\n", image);
    	  exit(-1);
  }

  Imlib_render (gImlib_data, img, img->rgb_width, img->rgb_height);
  
  i->image  = Imlib_move_image(gImlib_data, img);
  i->width  = img->rgb_width;
  i->height = img->rgb_height;

  return i;
} 

#define MWM_HINTS_DECORATIONS   (1L << 1)
#define PROP_MWM_HINTS_ELEMENTS 5
typedef struct _mwmhints {
  uint32_t flags;
  uint32_t functions;
  uint32_t decorations;
  int32_t  input_mode;
  uint32_t status;
} MWMHints;

void gui_open_panel () {
  GC                      gc;
  XSizeHints              hint;
  XSetWindowAttributes    attr;
  int                     screen;
  char                    title[] = {"Xine Panel"}; /* window-title     */
  input_plugin_t         *ip;
  int                     num_plugins, i;
  Atom                    prop;
  MWMHints                mwmhints;
  XClassHint             *xclasshint;


  if (gui_panel_win)
    return ; /* panel already open  - FIXME: bring to foreground */

  /*
   * load bg image before opening window, so we can determine it's size
   */

  if (!(gui_bg_image = Imlib_load_image(gImlib_data,
					gui_get_skinfile("BackGround")))) {
    fprintf(stderr, "xine-panel: couldn't find image %s\n", "bg.png");
    exit(-1);
  }

  screen = DefaultScreen(gDisplay);
  hint.x = config_file_lookup_int ("x_panel", 200);
  hint.y = config_file_lookup_int ("y_panel", 100);
  hint.width = gui_bg_image->rgb_width;
  hint.height = gui_bg_image->rgb_height;
  hint.flags = PPosition | PSize;
  
  attr.override_redirect = True;
  gui_panel_win = XCreateWindow (gDisplay, DefaultRootWindow(gDisplay), 
				 hint.x, hint.y, hint.width, hint.height, 0, 
				 CopyFromParent, CopyFromParent, 
				 CopyFromParent,
				 0, &attr);
  
  XSetStandardProperties(gDisplay, gui_panel_win, title, title,
			 None, NULL, 0, &hint);
  /*
   * wm, no border please
   */

  prop = XInternAtom(gDisplay, "_MOTIF_WM_HINTS", True);
  mwmhints.flags = MWM_HINTS_DECORATIONS;
  mwmhints.decorations = 0;

  XChangeProperty(gDisplay, gui_panel_win, prop, prop, 32,
                  PropModeReplace, (unsigned char *) &mwmhints,
                  PROP_MWM_HINTS_ELEMENTS);
  
  XSetTransientForHint (gDisplay, gui_panel_win, gVideoWin);

  /* set xclass */

  if((xclasshint = XAllocClassHint()) != NULL) {
    xclasshint->res_name = "Xine Panel";
    xclasshint->res_class = "Xine";
    XSetClassHint(gDisplay, gui_panel_win, xclasshint);
  }

  gc = XCreateGC(gDisplay, gui_panel_win, 0, 0);

  XSelectInput(gDisplay, gui_panel_win,
	       ButtonPressMask | ButtonReleaseMask | PointerMotionMask 
	       | KeyPressMask | ExposureMask | StructureNotifyMask);
  
  Imlib_apply_image(gImlib_data, gui_bg_image, gui_panel_win);
  XSync(gDisplay, False); 

  gui_make_window_dnd_aware (gui_panel_win);

  /*
   * Widget-list
   */
  gui_widget_list = (widget_list_t *) malloc (sizeof (widget_list_t));
  gui_widget_list->l = gui_list_new ();
  gui_widget_list->focusedWidget = NULL;
  gui_widget_list->pressedWidget = NULL;
  gui_widget_list->win           = gui_panel_win;
  gui_widget_list->gc            = gc;
 
  //PREV-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (
					  gui_get_skinX("Prev"),
					  gui_get_skinY("Prev"),
					  gui_nextprev, 
					  (void*)PREV, 
					  gui_get_skinfile("Prev")));//"prev.png"));

  //STOP-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("Stop"),
					  gui_get_skinY("Stop"), gui_stop, 
					  NULL, gui_get_skinfile("Stop")));
  
  //PLAY-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("Play"),
					  gui_get_skinY("Play"), gui_play,
					  NULL, gui_get_skinfile("Play")));

  // PAUSE-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("Pause"),
					  gui_get_skinY("Pause"), gui_pause,
					  NULL, gui_get_skinfile("Pause")));

  // NEXT-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("Next"),
					  gui_get_skinY("Next"),gui_nextprev,
					  (void*)NEXT, gui_get_skinfile("Next")));

  // EXIT-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("Exit"),
					  gui_get_skinY("Exit"), gui_exit,
					  NULL, gui_get_skinfile("Exit")));

  // Close-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("Close"),
					  gui_get_skinY("Close"), 
					  gui_toggle_panel_visibility,
					  NULL, gui_get_skinfile("Close"))); 
  // Fullscreen-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("FullScreen"),
					  gui_get_skinY("FullScreen"),
					  gui_toggle_fullscreen, NULL, 
					  gui_get_skinfile("FullScreen")));
  // Prev audio channel
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("AudioNext"),
					  gui_get_skinY("AudioNext"),
					  gui_change_audio_channel,
					  (void*)NEXT, 
					  gui_get_skinfile("AudioNext")));
  // Next audio channel
  gui_list_append_content (gui_widget_list->l, 
			   create_button (gui_get_skinX("AudioPrev"),
					  gui_get_skinY("AudioPrev"),
					  gui_change_audio_channel,
					  (void*)PREV, 
					  gui_get_skinfile("AudioPrev")));

  /* LABEL TITLE */
  gui_title_label = create_label (gui_get_skinX("TitleLabel"),
				  gui_get_skinY("TitleLabel"),
				  60, gui_filename, 
				  gui_get_skinfile("TitleLabel"));

  gui_list_append_content (gui_widget_list->l, gui_title_label);

  /* runtime label */
  gui_runtime_label = create_label (gui_get_skinX("TimeLabel"),
				    gui_get_skinY("TimeLabel"),
				    8, gui_runtime,
				    gui_get_skinfile("TimeLabel"));

  gui_list_append_content (gui_widget_list->l, gui_runtime_label);

  /* Audio channel label */
  gui_audiochan_label = create_label (gui_get_skinX("AudioLabel"),
				      gui_get_skinY("AudioLabel"),
				      3, gui_audiochan, 
				      gui_get_skinfile("AudioLabel"));

  gui_list_append_content (gui_widget_list->l, gui_audiochan_label);


  /* SLIDERS */
  gui_list_append_content (gui_widget_list->l,
			   (gui_slider_play = 
			    create_slider(HSLIDER,
					  gui_get_skinX("SliderBGPlay"),
					  gui_get_skinY("SliderBGPlay"),
					  0, 65535, 1, 
					  gui_get_skinfile("SliderBGPlay"),
					  gui_get_skinfile("SliderFGPlay"),
					  gui_slider, NULL)));

  /* FIXME: Need to implement mixer control
  gui_list_append_content (gui_widget_list->l,
			   (gui_slider_mixer = 
			    create_slider(VSLIDER, 
			    gui_get_skinX("SliderBGVol"),
			    gui_get_skinY("SliderBGVol"),
			    0, 100, 1, 
			    gui_get_skinfile("SliderBGVol"),
			    gui_get_skinfile("SliderFGVol"),
			    gui_slider, NULL)));
  */

  ip = xine_get_input_plugin_list (&num_plugins);

  xprintf (VERBOSE|GUI, "%d input plugins ...\n",num_plugins);

  if(VERBOSE|GUI)
    for (i=0; i<num_plugins; i++) {
      xprintf (VERBOSE|GUI, "plugin #%d : id=%s\n", i, ip->get_identifier());
      ip++;
    }
  
}

void gui_handle_event (XEvent *event) {
  XKeyEvent      mykeyevent;
  XExposeEvent  *myexposeevent;
  KeySym         mykey;
  static XEvent *old_event;
  char kbuf[256];
  int len;

  switch(event->type) {
  case Expose: {
    myexposeevent = (XExposeEvent *) event;

    if(event->xexpose.count == 0) {
      if (myexposeevent->window == gVideoWin)
	gVideoDriver->handle_event (event);
      else
	paint_widget_list (gui_widget_list);
    }
  }
  break;
    
  case MotionNotify:

    motion_notify_widget_list (gui_widget_list, event->xbutton.x, event->xbutton.y);
    /* if window-moving is enabled move the window */
    old_event = event;
    if (gui_move.enabled) {
      int x,y;
      x = (event->xmotion.x_root) + ( event->xmotion.x_root - old_event->xmotion.x_root) - gui_move.offset_x;
      y = (event->xmotion.y_root) + ( event->xmotion.y_root - old_event->xmotion.y_root) - gui_move.offset_y;

      XLockDisplay (gDisplay);
      XMoveWindow(gDisplay, gui_panel_win, x, y);
      XUnlockDisplay (gDisplay);

      config_file_set_int ("x_panel",x);
      config_file_set_int ("y_panel",y);

    }
    break;

  case MappingNotify:
    XLockDisplay (gDisplay);
    XRefreshKeyboardMapping((XMappingEvent *) event);
    XUnlockDisplay (gDisplay);
    break;

  case DestroyNotify:
    xine_exit ();
    gui_running = 0;
    break;

  case ButtonPress: {
    XButtonEvent *bevent = (XButtonEvent *) event;

    /* printf ("button: %d\n",bevent->button); */

    if ((bevent->button==3) && (bevent->window == gVideoWin)) {
      gui_toggle_panel_visibility (NULL, NULL);
      break;
    }
    
    /* if no widget is hit enable moving the window */
    if(bevent->window == gui_panel_win)
      gui_move.enabled = !click_notify_widget_list (gui_widget_list, 
						    event->xbutton.x, 
						    event->xbutton.y, 0);
    if (gui_move.enabled) {
      gui_move.offset_x = event->xbutton.x;
      gui_move.offset_y = event->xbutton.y;
    }
  }
  break;

  case ButtonRelease:
    click_notify_widget_list (gui_widget_list, event->xbutton.x, 
			      event->xbutton.y, 1);
    gui_move.enabled = 0; /* disable moving the window       */  
    break;

  case KeyPress:
    mykeyevent = event->xkey;
      
    XLockDisplay (gDisplay);
    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
    XUnlockDisplay (gDisplay);

    switch (mykey) {
    case XK_plus:
    case XK_KP_Add:
      gui_change_audio_channel(NULL, (void*)NEXT);
      break;
      
    case XK_minus:
    case XK_KP_Subtract:
      gui_change_audio_channel(NULL, (void*)PREV);
      break;
      
    case XK_P:
    case XK_p:
      gui_pause (NULL, NULL); 
      break;

    case XK_g:
    case XK_G:
      gui_toggle_panel_visibility (NULL, NULL);
      break;

    case XK_f:
    case XK_F:
      gui_toggle_fullscreen(NULL, NULL);
      break;

    case XK_a:
    case XK_A:
      gui_toggle_aspect();
      break;

    case XK_q:
    case XK_Q:
      gui_exit(NULL, NULL);
      break;

    case XK_Return:
    case XK_KP_Enter:
      gui_play(NULL, NULL);
      break;

    case XK_space:
      gui_pause(NULL, NULL);
      break;

    case XK_Control_L:
    case XK_Control_R:
      if(!gui_panel_visible) {
	cursor_visible = !cursor_visible;
	vo_display_cursor(cursor_visible);
      }
      break;

    case XK_Next:
      gui_nextprev(NULL, (void*)NEXT);
      break;

    case XK_Prior:
      gui_nextprev(NULL, (void*)PREV);
      break;
      
    case XK_1:
    case XK_KP_1:
      gui_set_current_position (6553);
      break;

    case XK_2:
    case XK_KP_2:
      gui_set_current_position (13107);
      break;

    case XK_3:
    case XK_KP_3:
      gui_set_current_position (19660);
      break;

    case XK_4:
    case XK_KP_4:
      gui_set_current_position (26214);
      break;

    case XK_5:
    case XK_KP_5:
      gui_set_current_position (32767);
      break;

    case XK_6:
    case XK_KP_6:
      gui_set_current_position (39321);
      break;

    case XK_7:
    case XK_KP_7:
      gui_set_current_position (45874);
      break;

    case XK_8:
    case XK_KP_8:
      gui_set_current_position (52428);
      break;

    case XK_9:
    case XK_KP_9:
      gui_set_current_position (58981);
      break;

    case XK_0:
    case XK_KP_0:
      gui_set_current_position (0);
      break;
      
    }

    break;

  case ConfigureNotify:
    /*  background */
    /*
    XLockDisplay (gDisplay);
    Imlib_apply_image(gImlib_data, gui_bg_image, gui_panel_win);
    XUnlockDisplay (gDisplay);
    paint_widget_list (gui_widget_list);
    XLockDisplay (gDisplay);
    XSync(gDisplay, False);
    XUnlockDisplay (gDisplay);
    */
    break;

  case ClientMessage:
    gui_dnd_process_client_message (event);
    break;
    /*
      default:
      printf("Got event: %i\n", event->type);
    */
  }

}

void gui_status_callback (int nStatus) {

  if (gui_ignore_status) 
    return;

  /* printf ("gui status callback : %d\n", nStatus); */

  if (gui_panel_visible) {
    sprintf (gui_runtime, "%02d:%02d:%02d", nVideoHours, nVideoMins, nVideoSecs);

    set_label (gui_runtime_label, gui_runtime);

    paint_label (gui_runtime_label, gui_panel_win, gui_widget_list->gc);

    if(update_slider > MAX_UPDSLD) {
      slider_set_pos(gui_widget_list, gui_slider_play, 
		     xine_get_current_position());
    
      update_slider = 0;
    }
    update_slider++;

  }

  if (nStatus == XINE_STOP) {
    gui_playlist_cur++;
    slider_reset(gui_widget_list, gui_slider_play);
    if (gui_playlist_cur < gui_playlist_num) {
      strcpy(gui_filename, gui_playlist [gui_playlist_cur]);
      set_label (gui_title_label, gui_filename);
      paint_label (gui_title_label, gui_panel_win, gui_widget_list->gc);
      xine_play ( gui_filename, 0 );
    } else {

      if(xine_user_pref & QUIT_ON_STOP)
	gui_exit(NULL, NULL);

      vo_set_logo_mode (1); 
    }
  }
}

#define SEND_KEVENT(key) {                                                    \
     static XEvent startevent;                                                \
     startevent.type = KeyPress;                                              \
     startevent.xkey.type = KeyPress;                                         \
     startevent.xkey.send_event = True;                                       \
     startevent.xkey.display = gDisplay;                                      \
     startevent.xkey.window = gui_panel_win;                                  \
     startevent.xkey.keycode = XKeysymToKeycode(gDisplay, key);               \
     XSendEvent(gDisplay, gui_panel_win, True, KeyPressMask, &startevent);    \
   }
void gui_start (int nfiles, char *filenames[]) {

  XEvent  myevent;
  int     i;

  /*
   * moving the window
   */

  gui_move.enabled  = 0;
  gui_move.offset_x = 0;
  gui_move.offset_y = 0; 

  for (i=0; i<nfiles; i++)
    gui_playlist[i]     = filenames[i];
  gui_playlist_num = nfiles; 
  gui_playlist_cur = 0;

  if (nfiles)
    strcpy(gui_filename, gui_playlist [gui_playlist_cur]);
  else 
    sprintf(gui_filename, "DROP A FILE ON XINE");

  sprintf (gui_runtime, "%02d:%02d:%02d", nVideoHours, nVideoMins, nVideoSecs);
  sprintf (gui_audiochan, "%3d", curAudioTrack);

  XLockDisplay (gDisplay);
  
  /*
   * setup panel
   */

  gui_open_panel ();

  if (config_file_lookup_int ("open_panel", 1)) {
    XMapRaised(gDisplay, gui_panel_win); 
    gui_panel_visible = 1;
  } else
    gui_panel_visible = 0;

  vo_display_cursor(gui_panel_visible);
  cursor_visible = gui_panel_visible;
  XUnlockDisplay (gDisplay);
  
  /*  The user request "play on start" */
  if(xine_user_pref & PLAY_ON_START) {
    /* The user want to hide control panel */
    if(gui_panel_visible && (xine_user_pref & HIDEGUI_ON_START))
      SEND_KEVENT(XK_G);

    /* The user want to see in fullscreen mode */
    if(xine_user_pref & FULL_ON_START)
      SEND_KEVENT(XK_F);

    SEND_KEVENT(XK_Return);
    xine_user_pref |= PLAYED_ON_START;
  }

  /*
   * event loop
   */

  gui_running = 1;
  while (gui_running) {
    XNextEvent (gDisplay, &myevent) ;

    gui_handle_event (&myevent) ;
  }
}
