/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: input_plugin.h,v 1.10 2000/12/12 22:05:09 heikos Exp $
 */

#ifndef HAVE_INPUT_PLUGIN_H
#define HAVE_INPUT_PLUGIN_H

#include <inttypes.h>
#include <sys/types.h>

#define INPUT_PLUGIN_MAX    127

typedef struct input_plugin_s
{

  /*
   * handle of plugin
   */
  void *handle;

  /*
   * filename of plugin
   */
  char *filename;
 
  /*
   * init plugin
   */
  void (*init) (void);

  /*
   * open input MRL - return 1 if succ
   */

  int (*open) (char *mrl);

  /*
   * read nlen bytes, return number of bytes read
   */
  uint32_t (*read)(char *buf, uint32_t nlen);

  /*
   * seek position, return new position (=> way to determine cur pos)
   */
  off_t (*seek) (off_t offset, int origin);

  /*
   * return length of input (-1 => unlimited, e.g. stream)
   */

  off_t (*get_length) (void);

  /*
   * return capabilities of input source
   */

  uint32_t (*get_capabilities) (void);

  /*
   * ls function
   * return value: 0 => filename is a file, 1=> filename is a dir
   */

  int (*get_dir) (char *filename, char **flist, int *nFiles);

  /*
   * return block size of input source (if supported, 0 otherwise)
   */

  uint32_t (*get_blocksize) (void);
 
  /*
   * close input source
   */
  
  void (*close) (void);

  /*
   * return human readable identifier for this plugin
   */

  char* (*get_identifier) (void);

} input_plugin_t;

#define INPUT_CAP_SEEKABLE 1
#define INPUT_CAP_BLOCK    2


#endif
