/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: metronom.h,v 1.1 2000/10/30 22:42:58 guenter Exp $
 *
 * metronom: general pts => virtual calculation/assoc
 *                   
 * virtual pts: unit 1/90000 sec, always increasing
 *              can be used for synchronization
 *              video/audio frame with same pts also have same vpts
 *              but pts is likely to differ from vpts
 *
 */


#ifndef HAVE_METRONOM_H
#define HAVE_METRONOM_H

#include <inttypes.h>

/*
 * clear all cached data, reset current vpts ... called if new input
 * file is reached
 */

void metronom_reset ();

/*
 * called by video output driver to inform metronom about current framerate
 *
 * parameter pts_per_frame : frame display duration in 1/90000 sec
 */
void metronom_set_video_rate (uint32_t pts_per_frame);

/*
 * called by audio output driver to inform metronom about current audio
 * bitrate
 *
 * parameter pts_per_smpls : 1/90000 sec per 65536 samples
 */
void metronom_set_audio_rate (uint32_t pts_per_smpls);

/*
 * called by video output driver for *every* frame
 *
 * parameter pts: pts for frame if known, 0 otherwise
 *
 * return value: virtual pts for frame
 *
 */

uint32_t metronom_got_video_frame (uint32_t pts);

/*
 * called by audio output driver whenever audio samples are delivered to it
 *
 * parameter pts      : pts for audio data if known, 0 otherwise
 *           nsamples : number of samples delivered
 *
 * return value: virtual pts for audio data
 *
 */

uint32_t metronom_got_audio_samples (uint32_t pts, uint32_t nsamples); 

#endif
